# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-11-16 15:51
from __future__ import unicode_literals

from django.db import migrations, models
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('edx_proctoring', '0007_proctoredexam_backend'),
    ]

    operations = [
        migrations.AddField(
            model_name='proctoredexamreviewpolicy',
            name='rules',
            field=jsonfield.fields.JSONField(null=True),
        ),
        migrations.AddField(
            model_name='proctoredexamreviewpolicyhistory',
            name='rules',
            field=jsonfield.fields.JSONField(null=True),
        ),
        migrations.AlterField(
            model_name='proctoredexamreviewpolicy',
            name='review_policy',
            field=models.TextField(default=b''),
        ),
    ]
