{% load i18n %}
<div class="success sequence proctored-exam passed" data-exam-id="{{exam_id}}">
  <h3>
    {% blocktrans %}
      Your proctoring session was reviewed successfully. Go to your progress page to view your exam grade.
    {% endblocktrans %}
  </h3>

    {% include 'proctored_exam/visit_exam_content.html' %}
</div>
{% include 'proctored_exam/footer.html' %}
