{% load i18n %}
<div class="footer-sequence">
  <h4> {% trans "Can I request additional time to complete my exam?" %} </h4>
    <p>{% blocktrans %}
      If you have disabilities,
      you might be eligible for an additional time allowance on timed exams.
      Ask your course team for information about additional time allowances.
    {% endblocktrans %}
  </p>
</div>
